/*  @(#)05  1.17 src/com/tivoli/pd/jazn/demo/PDPermissionDemo.java, pd.jazn, am610, 080214a 07/03/15 14:13:23 @(#)  */

/*
 *
 *
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2003
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * File Name: PDPermissionDemo.java
 */



import java.util.ArrayList;
import java.util.Set;
import java.net.*;
import java.io.*;
import com.tivoli.pd.jazn.*;
import com.tivoli.pd.jadmin.PDActionGroup;
import com.tivoli.pd.jadmin.PDAction;
import com.tivoli.pd.jutil.PDAttrs;
import com.tivoli.pd.jutil.PDException;
import com.tivoli.pd.jutil.PDMessages;
import com.tivoli.pd.jutil.PDStatics;

/**
 * IBM Tivoli Access Manager for e-business PDPermissionDemo class.
 *
 * <p>
 * This class demonstrates the use of the authorization related
 * Access Manager classes.
 * <p>
 * To use the demo you must have a configuration file.  This file is the same
 * as the one specified to the com.tivoli.pd.jcfg.SvrSslCfg
 * configuration program.
 * <p>
 * The demo will create a PDAuthorizationContext using the configuraton data.
 * The PDAuthorizationContext represents the application's configuration
 * and connection to the Access Manager secure domain and the servers therein.
 * The context is used as well to retrieve the list of valid actions in
 * the Access Manager secure domain.
 * <p>
 * The demo will then repeatedly prompt for the user, resource, and actions to
 * test.  Each test will result in a true or false result which indicates
 * the user's authorization to perform the specified actions on the specified
 * resource.
 * <p>
 * You may quit the demo by specifying "quit" at any of the prompts.
 */

public final class PDPermissionDemo
{

    private final String PDPermissionDemo_java_sourceCodeID="$Id: @(#)05  1.17 src/com/tivoli/pd/jazn/demo/PDPermissionDemo.java, pd.jazn, am610, 080214a 07/03/15 14:13:23 @(#) $";
    public static final String exitString = "quit";
    public static final String configFilePrompt = "configuration file";
    public static final String userPrompt = "user";
    public static final String userDefault = "unauthenticated";
    public static final String passwordPrompt = "password (optional)";
    public static final String passwordDefault = "";
    public static final String resourcePrompt = "resource";
    public static final String resourceDefault = "/";
    public static final String actionsPrompt = "actions";
    public static final String actionsDefault = "T";
    public static final String availableActionsPrompt = "[available actions]:";

    String configFile_ = null;
    String user_ = null;
    String password_ = null;
    String resource_ = null;
    String actions_ = null;
    boolean exit_ = false;


    /**
     * Prompts for input.
     *
     * @param reader  the reader used to retrieve input.
     * @param prompt  string used for prompt when retrieving input.
     * @param def     string default returned when no input is provided.
     *
     * @return        string input.  This is the value specified by the default
     *                string when no input is provided.
     */
    private String prompt(BufferedReader reader,
                          String prompt,
                          String def) throws java.io.IOException
    {
        String result = null;
        System.out.print(prompt);

        if (def != null)
        {
            System.out.print(" [");
            System.out.print(def);
            System.out.print("]");
        }

        System.out.print(":  ");
        result = reader.readLine();
        if (result.length() == 0)
        {
            result = def;
        }
        return result;
    }

    /**
     * Constructs a PDAuthorizationContext based on configuration file input.
     * This method will continue to request a valid configuration file until
     * either a PDAuthorizationContext can be successfully constructed or
     * the demo is exited.
     *
     * @param reader  the reader used to retrieve the configuration file
     *                input.  The configuration file specified is used
     *                to construct a PDAuthorizationContext.
     *
     * @return        PDAuthorizationContext based on configuration file
     *                specified.
     */
    private PDAuthorizationContext getAuthorizationContext(BufferedReader reader)
    {
        PDAuthorizationContext ctxt=null;
        while ((!exit()) && (ctxt == null))
        {
            try
            {
                configFile_ = prompt(reader, configFilePrompt, configFile_);
                if (!exit(configFile_))
                {
                    File cFile = new File(configFile_);
                    if (cFile.exists())
                    {
                        if (cFile.isFile())
                        {
                            String spec = new String("file:///").concat(configFile_);
                            URL configURL = new URL(spec);
                            ctxt = new PDAuthorizationContext(configURL);
                        }
                        else
                        {
                            System.out.println("Specified configuration file is not a file.");
                        }
                    }
                    else
                    {
                        System.out.println("Specified configuration file does not exist.");
                    }
                }
            }
            catch (Exception e)
            {
                e.printStackTrace();
                ctxt = null;
            }
        }
        return ctxt;
    }

    /**
     * Constructs the list of valid actions in this Access Manager secure
     * domain.  This list of actions is represented by a string.  Each action
     * is a single character a-z or A-Z and represents a mode of access. For
     * example "create", "modify", and "delete" could each be considered a
     * mode of access.
     * <p>
     * The primary group of actions defined by Access Manager are in an
     * "unnamed" group and appear first in the list.  Additional action
     * groups and their actions follow.  This is the form supported by the
     * Access Manager authorization classes.  The format is as follows:
     * <p>
     * a-z,A-Z[action group name]a-z,A-Z[action group name]a-z,A-Z...
     *
     * @param ctxt  the PDAuthorizationContext representing the
     *              application's configuration and connection
     *              to the secure domain.
     *
     * @return      string which contains all of the possible action groups
     *              and actions defined for the Access Manager secure
     *              domain.  If the list of valid actions are not available
     *              an empty string is returned.
     */
    private String getActions(PDAuthorizationContext ctxt)
    {
        String actions = new String();
        PDMessages pdms = new PDMessages();

        try
        {
            ArrayList actionGroupList = PDActionGroup.listActionGroups(ctxt, pdms);

            for(int i=0; i<actionGroupList.size(); i++)
            {
                String actionGroupName = (String)actionGroupList.get(i);
                ArrayList actionList = PDAction.listActions(ctxt, actionGroupName, pdms);
                if (actionList.size() > 0)
                {
                    if (!actionGroupName.equals(PDActionGroup.PDACTIONGROUP_PRIMARY_NAME))
                    {
                        actions += "["+actionGroupName+"]";
                    }
                    for (int j=0; j<actionList.size(); j++)
                    {
                        String action = (String)actionList.get(j);
                        actions += action;
                    }
                }
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
            actions = "";
        }

        return actions;
    }
    /**
     * Constructs a PDPrincipal object for the specified user.  The PDPrincipal
     * object represents a user identity and it's credentials in the Access
     * Manager secure domain.  The user credential's consist of it's group
     * memberships and any additional data that has been added to the
     * credential.
     * <p>
     * This method will continue to request a valid user until either a
     * PDPrincipal can be successfully constructed or the demo is exited.
     *
     * @param reader  the reader used to retrieve the Access Manager user
     *                identity input.  The user identity specified is used
     *                to construct a PDPrincipal.
     * @param ctxt    the PDAuthorizationContext representing the
     *                application's configuration and connection
     *                to the secure domain.
     *
     * @return        PDPrinicpal based on user identity specified.
     */
    private PDPrincipal getPrincipal(BufferedReader reader, PDAuthorizationContext ctxt)
    {
        PDPrincipal principal = null;
        while ((!exit()) && (principal == null))
        {
            try
            {
                user_ = prompt(reader, userPrompt, user_);
                if (!exit(user_))
                {
                    // default user is unauth
                    if (user_.toLowerCase().startsWith("unauth"))
                    {
                        principal = new PDPrincipal(ctxt);
                    }
                    else
                    {
                        password_ = prompt(reader, passwordPrompt, password_);
                        if (!exit(password_))
                        {
                            if (password_.length() == 0)
                            {
                                principal = new PDPrincipal(ctxt, user_);
                            }
                            else
                            {
                                principal = new PDPrincipal(ctxt, user_, password_.toCharArray());
                            }
                        }
                    }
                }
            }
            catch (Exception e)
            {
                e.printStackTrace();
                principal = null;
            }
        }
        return principal;
    }

    /**
     * Constructs a PDPermission object for the specified resource and actions.
     * The PDPermission object is the combination of a resource and actions.
     * The resource is a string which identifies the accessed resource.
     * The actions are a string that identifies the modes of access.
     * <p>
     * This method will continue to request a valid resource and valid
     * actions until either a PDPermission can be successfully constructed
     * or the demo is exited.
     *
     * @param reader           the reader used to retrieve the resource and
     *                         actions input.  The resource and actions
     *                         specified are used to construct a
     *                         PDPermission.
     * @param ctxt             the PDAuthorizationContext representing the
     *                         application's configuration and connection
     *                         to the secure domain.
     * @param availableActions the list of valid action groups and actions
     *                         defined in Access Manager domain.
     *
     * @return                 PDPermission based on resource and actions
     *                         specified.
     */
    private PDPermission getPermission(BufferedReader reader,
                                       PDAuthorizationContext ctxt,
                                       String availableActions)
    {
        PDPermission permission = null;
        while ((!exit()) && (permission == null))
        {
            try
            {
                resource_ = prompt(reader, resourcePrompt, resource_);
                if (!exit(resource_))
                {
                    System.out.print(availableActionsPrompt);
                    System.out.println(availableActions);
                    actions_ = prompt(reader, actionsPrompt, actions_);
                    if (!exit(actions_))
                    {
                        permission = new PDPermission(ctxt, resource_, actions_);
                    }
                }
            }
            catch (Exception e)
            {
                e.printStackTrace();
                permission = null;
            }
        }
        return permission;
    }

    /**
     * Constructs a PDPermissionDemo object.
     */
    public PDPermissionDemo()
    {
        user_ = userDefault;
        password_ = passwordDefault;
        resource_ = resourceDefault;
        actions_ = actionsDefault;
    }

    /**
     * Returns a boolean to indicate the demo should exit.
     */
    public boolean exit()
    {
        return exit_;
    }

    /**
     * Tests an input string to determine if the demo should exit.
     *
     * @param input  string to test.  A case insensitive comparison is done
     *               against the string "quit" to determine whether the
     *               the demo should exit.
     *
     * @return       boolean to indicate the demo should exit.
     */
    private boolean exit(String input)
    {
        if (input.equalsIgnoreCase(exitString))
        {
            exit_ = true;
        }
        else
        {
            exit_ = false;
        }
        return exit_;
    }

    /**
     * Main program.
     */
    public static void main (String [] argv)
    {
        // construct demo object.
        PDPermissionDemo demo = new PDPermissionDemo();

        // construct reader from standard input stream.
        InputStreamReader input = new InputStreamReader(System.in);
        BufferedReader reader = new BufferedReader(input);

        // construct authorization context based on input configuration file.
        PDAuthorizationContext ctxt = demo.getAuthorizationContext(reader);

        // retrieve list of valid action groups and actions.
        String availableActions = demo.getActions(ctxt);

        // while no indication to exit
        while (!demo.exit())
        {
            try
            {
                // construct principal from input user identity
                PDPrincipal principal = demo.getPrincipal(reader, ctxt);
                if (!demo.exit())
                {
                    // construct permission from input resource and actions
                    PDPermission permission = demo.getPermission(reader, ctxt, availableActions);
                    if (!demo.exit())
                    {
                        // construct output attributes container
                        PDAttrs attrs = new PDAttrs(ctxt, true);

                        // test access
                        System.out.print("Authorized:  ");
                        System.out.println(permission.implies(ctxt, principal, null, attrs));

                        // if any output attributes were returned, then display
                        Set s = attrs.keySet();
                        if (!s.isEmpty())
                        {
                            System.out.println("Attributes returned:  ");
                            System.out.println(attrs);
                        }
                    }
                }
            }
            catch (PDException pde)
            {
                pde.printStackTrace();
            }
        }

        // close the context
        if (ctxt != null)
        {
            try
            {
                ctxt.close();
            }
            catch (PDException pde)
            {
                pde.printStackTrace();
            }
        }
    }
}
